# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActionCheckRegistrationStatusResponse", "Data"]


class Data(BaseModel):
    ip_address: Optional[str] = None
    """The ip used during the SIP connection"""

    last_registration: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was last updated."""

    port: Optional[int] = None
    """The port of the SIP connection"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    sip_username: Optional[str] = None
    """The user name of the SIP connection"""

    status: Optional[Literal["Not Applicable", "Not Registered", "Failed", "Expired", "Registered", "Unregistered"]] = (
        None
    )
    """The current registration status of your SIP connection"""

    transport: Optional[str] = None
    """The protocol of the SIP connection"""

    user_agent: Optional[str] = None
    """The user agent of the SIP connection"""


class ActionCheckRegistrationStatusResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.assistants import canary_deploy_create_params, canary_deploy_update_params
from ....types.ai.assistants.version_config_param import VersionConfigParam
from ....types.ai.assistants.canary_deploy_response import CanaryDeployResponse

__all__ = ["CanaryDeploysResource", "AsyncCanaryDeploysResource"]


class CanaryDeploysResource(SyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> CanaryDeploysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CanaryDeploysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CanaryDeploysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CanaryDeploysResourceWithStreamingResponse(self)

    def create(
        self,
        assistant_id: str,
        *,
        versions: Iterable[VersionConfigParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to create a canary deploy configuration for an assistant.

        Creates a new canary deploy configuration with multiple version IDs and their
        traffic percentages for A/B testing or gradual rollouts of assistant versions.

        Args:
          versions: List of version configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._post(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            body=maybe_transform({"versions": versions}, canary_deploy_create_params.CanaryDeployCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    def retrieve(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to get a canary deploy configuration for an assistant.

        Retrieves the current canary deploy configuration with all version IDs and their
        traffic percentages for the specified assistant.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._get(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    def update(
        self,
        assistant_id: str,
        *,
        versions: Iterable[VersionConfigParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to update a canary deploy configuration for an assistant.

        Updates the existing canary deploy configuration with new version IDs and
        percentages. All old versions and percentages are replaces by new ones from this
        request.

        Args:
          versions: List of version configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._put(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            body=maybe_transform({"versions": versions}, canary_deploy_update_params.CanaryDeployUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    def delete(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Endpoint to delete a canary deploy configuration for an assistant.

        Removes all canary deploy configurations for the specified assistant.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCanaryDeploysResource(AsyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> AsyncCanaryDeploysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCanaryDeploysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCanaryDeploysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCanaryDeploysResourceWithStreamingResponse(self)

    async def create(
        self,
        assistant_id: str,
        *,
        versions: Iterable[VersionConfigParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to create a canary deploy configuration for an assistant.

        Creates a new canary deploy configuration with multiple version IDs and their
        traffic percentages for A/B testing or gradual rollouts of assistant versions.

        Args:
          versions: List of version configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._post(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            body=await async_maybe_transform(
                {"versions": versions}, canary_deploy_create_params.CanaryDeployCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    async def retrieve(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to get a canary deploy configuration for an assistant.

        Retrieves the current canary deploy configuration with all version IDs and their
        traffic percentages for the specified assistant.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._get(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    async def update(
        self,
        assistant_id: str,
        *,
        versions: Iterable[VersionConfigParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CanaryDeployResponse:
        """
        Endpoint to update a canary deploy configuration for an assistant.

        Updates the existing canary deploy configuration with new version IDs and
        percentages. All old versions and percentages are replaces by new ones from this
        request.

        Args:
          versions: List of version configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._put(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            body=await async_maybe_transform(
                {"versions": versions}, canary_deploy_update_params.CanaryDeployUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CanaryDeployResponse,
        )

    async def delete(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Endpoint to delete a canary deploy configuration for an assistant.

        Removes all canary deploy configurations for the specified assistant.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/assistants/{assistant_id}/canary-deploys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CanaryDeploysResourceWithRawResponse:
    def __init__(self, canary_deploys: CanaryDeploysResource) -> None:
        self._canary_deploys = canary_deploys

        self.create = to_raw_response_wrapper(
            canary_deploys.create,
        )
        self.retrieve = to_raw_response_wrapper(
            canary_deploys.retrieve,
        )
        self.update = to_raw_response_wrapper(
            canary_deploys.update,
        )
        self.delete = to_raw_response_wrapper(
            canary_deploys.delete,
        )


class AsyncCanaryDeploysResourceWithRawResponse:
    def __init__(self, canary_deploys: AsyncCanaryDeploysResource) -> None:
        self._canary_deploys = canary_deploys

        self.create = async_to_raw_response_wrapper(
            canary_deploys.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            canary_deploys.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            canary_deploys.update,
        )
        self.delete = async_to_raw_response_wrapper(
            canary_deploys.delete,
        )


class CanaryDeploysResourceWithStreamingResponse:
    def __init__(self, canary_deploys: CanaryDeploysResource) -> None:
        self._canary_deploys = canary_deploys

        self.create = to_streamed_response_wrapper(
            canary_deploys.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            canary_deploys.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            canary_deploys.update,
        )
        self.delete = to_streamed_response_wrapper(
            canary_deploys.delete,
        )


class AsyncCanaryDeploysResourceWithStreamingResponse:
    def __init__(self, canary_deploys: AsyncCanaryDeploysResource) -> None:
        self._canary_deploys = canary_deploys

        self.create = async_to_streamed_response_wrapper(
            canary_deploys.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            canary_deploys.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            canary_deploys.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            canary_deploys.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Messaging10dlcGetEnumResponse", "EnumPaginatedResponse"]


class EnumPaginatedResponse(BaseModel):
    page: int

    records: List[Dict[str, object]]

    total_records: int = FieldInfo(alias="totalRecords")


Messaging10dlcGetEnumResponse: TypeAlias = Union[
    List[str], List[Dict[str, object]], Dict[str, object], Dict[str, object], EnumPaginatedResponse
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.actions import purchase_create_params
from ...types.actions.purchase_create_response import PurchaseCreateResponse

__all__ = ["PurchaseResource", "AsyncPurchaseResource"]


class PurchaseResource(SyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> PurchaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PurchaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PurchaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PurchaseResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        amount: int,
        product: str | Omit = omit,
        sim_card_group_id: str | Omit = omit,
        status: Literal["enabled", "disabled", "standby"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        whitelabel_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PurchaseCreateResponse:
        """
        Purchases and registers the specified amount of eSIMs to the current user's
        account.<br/><br/> If <code>sim_card_group_id</code> is provided, the eSIMs will
        be associated with that group. Otherwise, the default group for the current user
        will be used.<br/><br/>

        Args:
          amount: The amount of eSIMs to be purchased.

          product: Type of product to be purchased, specify "whitelabel" to use a custom SPN

          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          status: Status on which the SIM cards will be set after being successfully registered.

          tags: Searchable tags associated with the SIM cards

          whitelabel_name: Service Provider Name (SPN) for the Whitelabel eSIM product. It will be
              displayed as the mobile service name by operating systems of smartphones. This
              parameter must only contain letters, numbers and whitespaces.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/actions/purchase/esims",
            body=maybe_transform(
                {
                    "amount": amount,
                    "product": product,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                    "whitelabel_name": whitelabel_name,
                },
                purchase_create_params.PurchaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseCreateResponse,
        )


class AsyncPurchaseResource(AsyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> AsyncPurchaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPurchaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPurchaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPurchaseResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        amount: int,
        product: str | Omit = omit,
        sim_card_group_id: str | Omit = omit,
        status: Literal["enabled", "disabled", "standby"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        whitelabel_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PurchaseCreateResponse:
        """
        Purchases and registers the specified amount of eSIMs to the current user's
        account.<br/><br/> If <code>sim_card_group_id</code> is provided, the eSIMs will
        be associated with that group. Otherwise, the default group for the current user
        will be used.<br/><br/>

        Args:
          amount: The amount of eSIMs to be purchased.

          product: Type of product to be purchased, specify "whitelabel" to use a custom SPN

          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          status: Status on which the SIM cards will be set after being successfully registered.

          tags: Searchable tags associated with the SIM cards

          whitelabel_name: Service Provider Name (SPN) for the Whitelabel eSIM product. It will be
              displayed as the mobile service name by operating systems of smartphones. This
              parameter must only contain letters, numbers and whitespaces.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/actions/purchase/esims",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "product": product,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                    "whitelabel_name": whitelabel_name,
                },
                purchase_create_params.PurchaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseCreateResponse,
        )


class PurchaseResourceWithRawResponse:
    def __init__(self, purchase: PurchaseResource) -> None:
        self._purchase = purchase

        self.create = to_raw_response_wrapper(
            purchase.create,
        )


class AsyncPurchaseResourceWithRawResponse:
    def __init__(self, purchase: AsyncPurchaseResource) -> None:
        self._purchase = purchase

        self.create = async_to_raw_response_wrapper(
            purchase.create,
        )


class PurchaseResourceWithStreamingResponse:
    def __init__(self, purchase: PurchaseResource) -> None:
        self._purchase = purchase

        self.create = to_streamed_response_wrapper(
            purchase.create,
        )


class AsyncPurchaseResourceWithStreamingResponse:
    def __init__(self, purchase: AsyncPurchaseResource) -> None:
        self._purchase = purchase

        self.create = async_to_streamed_response_wrapper(
            purchase.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import fqdn_list_params, fqdn_create_params, fqdn_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..types.fqdn import Fqdn
from .._base_client import AsyncPaginator, make_request_options
from ..types.fqdn_create_response import FqdnCreateResponse
from ..types.fqdn_delete_response import FqdnDeleteResponse
from ..types.fqdn_update_response import FqdnUpdateResponse
from ..types.fqdn_retrieve_response import FqdnRetrieveResponse

__all__ = ["FqdnsResource", "AsyncFqdnsResource"]


class FqdnsResource(SyncAPIResource):
    """FQDN operations"""

    @cached_property
    def with_raw_response(self) -> FqdnsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return FqdnsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FqdnsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return FqdnsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        connection_id: str,
        dns_record_type: str,
        fqdn: str,
        port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnCreateResponse:
        """
        Create a new FQDN object.

        Args:
          connection_id: ID of the FQDN connection to which this IP should be attached.

          dns_record_type: The DNS record type for the FQDN. For cases where a port is not set, the DNS
              record type must be 'srv'. For cases where a port is set, the DNS record type
              must be 'a'. If the DNS record type is 'a' and a port is not specified, 5060
              will be used.

          fqdn: FQDN represented by this resource.

          port: Port to use when connecting to this FQDN.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/fqdns",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "dns_record_type": dns_record_type,
                    "fqdn": fqdn,
                    "port": port,
                },
                fqdn_create_params.FqdnCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnRetrieveResponse:
        """
        Return the details regarding a specific FQDN.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fqdns/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        connection_id: str | Omit = omit,
        dns_record_type: str | Omit = omit,
        fqdn: str | Omit = omit,
        port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnUpdateResponse:
        """
        Update the details of a specific FQDN.

        Args:
          connection_id: ID of the FQDN connection to which this IP should be attached.

          dns_record_type: The DNS record type for the FQDN. For cases where a port is not set, the DNS
              record type must be 'srv'. For cases where a port is set, the DNS record type
              must be 'a'. If the DNS record type is 'a' and a port is not specified, 5060
              will be used.

          fqdn: FQDN represented by this resource.

          port: Port to use when connecting to this FQDN.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/fqdns/{id}",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "dns_record_type": dns_record_type,
                    "fqdn": fqdn,
                    "port": port,
                },
                fqdn_update_params.FqdnUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnUpdateResponse,
        )

    def list(
        self,
        *,
        filter: fqdn_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[Fqdn]:
        """
        Get all FQDNs belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_id], filter[fqdn], filter[port], filter[dns_record_type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/fqdns",
            page=SyncDefaultFlatPagination[Fqdn],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    fqdn_list_params.FqdnListParams,
                ),
            ),
            model=Fqdn,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnDeleteResponse:
        """
        Delete an FQDN.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/fqdns/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnDeleteResponse,
        )


class AsyncFqdnsResource(AsyncAPIResource):
    """FQDN operations"""

    @cached_property
    def with_raw_response(self) -> AsyncFqdnsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFqdnsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFqdnsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncFqdnsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        connection_id: str,
        dns_record_type: str,
        fqdn: str,
        port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnCreateResponse:
        """
        Create a new FQDN object.

        Args:
          connection_id: ID of the FQDN connection to which this IP should be attached.

          dns_record_type: The DNS record type for the FQDN. For cases where a port is not set, the DNS
              record type must be 'srv'. For cases where a port is set, the DNS record type
              must be 'a'. If the DNS record type is 'a' and a port is not specified, 5060
              will be used.

          fqdn: FQDN represented by this resource.

          port: Port to use when connecting to this FQDN.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/fqdns",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "dns_record_type": dns_record_type,
                    "fqdn": fqdn,
                    "port": port,
                },
                fqdn_create_params.FqdnCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnRetrieveResponse:
        """
        Return the details regarding a specific FQDN.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fqdns/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        connection_id: str | Omit = omit,
        dns_record_type: str | Omit = omit,
        fqdn: str | Omit = omit,
        port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnUpdateResponse:
        """
        Update the details of a specific FQDN.

        Args:
          connection_id: ID of the FQDN connection to which this IP should be attached.

          dns_record_type: The DNS record type for the FQDN. For cases where a port is not set, the DNS
              record type must be 'srv'. For cases where a port is set, the DNS record type
              must be 'a'. If the DNS record type is 'a' and a port is not specified, 5060
              will be used.

          fqdn: FQDN represented by this resource.

          port: Port to use when connecting to this FQDN.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/fqdns/{id}",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "dns_record_type": dns_record_type,
                    "fqdn": fqdn,
                    "port": port,
                },
                fqdn_update_params.FqdnUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnUpdateResponse,
        )

    def list(
        self,
        *,
        filter: fqdn_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Fqdn, AsyncDefaultFlatPagination[Fqdn]]:
        """
        Get all FQDNs belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_id], filter[fqdn], filter[port], filter[dns_record_type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/fqdns",
            page=AsyncDefaultFlatPagination[Fqdn],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    fqdn_list_params.FqdnListParams,
                ),
            ),
            model=Fqdn,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnDeleteResponse:
        """
        Delete an FQDN.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/fqdns/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnDeleteResponse,
        )


class FqdnsResourceWithRawResponse:
    def __init__(self, fqdns: FqdnsResource) -> None:
        self._fqdns = fqdns

        self.create = to_raw_response_wrapper(
            fqdns.create,
        )
        self.retrieve = to_raw_response_wrapper(
            fqdns.retrieve,
        )
        self.update = to_raw_response_wrapper(
            fqdns.update,
        )
        self.list = to_raw_response_wrapper(
            fqdns.list,
        )
        self.delete = to_raw_response_wrapper(
            fqdns.delete,
        )


class AsyncFqdnsResourceWithRawResponse:
    def __init__(self, fqdns: AsyncFqdnsResource) -> None:
        self._fqdns = fqdns

        self.create = async_to_raw_response_wrapper(
            fqdns.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            fqdns.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            fqdns.update,
        )
        self.list = async_to_raw_response_wrapper(
            fqdns.list,
        )
        self.delete = async_to_raw_response_wrapper(
            fqdns.delete,
        )


class FqdnsResourceWithStreamingResponse:
    def __init__(self, fqdns: FqdnsResource) -> None:
        self._fqdns = fqdns

        self.create = to_streamed_response_wrapper(
            fqdns.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            fqdns.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            fqdns.update,
        )
        self.list = to_streamed_response_wrapper(
            fqdns.list,
        )
        self.delete = to_streamed_response_wrapper(
            fqdns.delete,
        )


class AsyncFqdnsResourceWithStreamingResponse:
    def __init__(self, fqdns: AsyncFqdnsResource) -> None:
        self._fqdns = fqdns

        self.create = async_to_streamed_response_wrapper(
            fqdns.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            fqdns.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            fqdns.update,
        )
        self.list = async_to_streamed_response_wrapper(
            fqdns.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            fqdns.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SimCardDeleteParams"]


class SimCardDeleteParams(TypedDict, total=False):
    report_lost: bool
    """Enables deletion of disabled eSIMs that can't be uninstalled from a device.

    This is irreversible and the eSIM cannot be re-registered.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.feature import Feature
from .shared.cost_information import CostInformation
from .shared.region_information import RegionInformation
from .shared.available_phone_numbers_metadata import AvailablePhoneNumbersMetadata

__all__ = ["AvailablePhoneNumberBlockListResponse", "Data"]


class Data(BaseModel):
    cost_information: Optional[CostInformation] = None

    features: Optional[List[Feature]] = None

    phone_number: Optional[str] = None

    range: Optional[int] = None

    record_type: Optional[Literal["available_phone_number_block"]] = None

    region_information: Optional[List[RegionInformation]] = None


class AvailablePhoneNumberBlockListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[AvailablePhoneNumbersMetadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .integration_connection import IntegrationConnection as IntegrationConnection
from .connection_list_response import ConnectionListResponse as ConnectionListResponse
from .connection_retrieve_response import ConnectionRetrieveResponse as ConnectionRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BrandOptionalAttributes"]


class BrandOptionalAttributes(BaseModel):
    tax_exempt_status: Optional[str] = FieldInfo(alias="taxExemptStatus", default=None)
    """The tax exempt status of the brand"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .anchorsite_override import AnchorsiteOverride

__all__ = ["FaxApplication", "Inbound", "Outbound"]


class Inbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the number of concurrent inbound calls to phone
    numbers associated with this connection.
    """

    sip_subdomain: Optional[str] = None
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Optional[Literal["only_my_connections", "from_anyone"]] = None
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """


class Outbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the number of concurrent outbound calls to phone
    numbers associated with this connection.
    """

    outbound_voice_profile_id: Optional[str] = None
    """Identifies the associated outbound voice profile."""


class FaxApplication(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the resource."""

    active: Optional[bool] = None
    """Specifies whether the connection can be used."""

    anchorsite_override: Optional[AnchorsiteOverride] = None
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    application_name: Optional[str] = None
    """A user-assigned name to help manage the application."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    inbound: Optional[Inbound] = None

    outbound: Optional[Outbound] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    tags: Optional[List[str]] = None
    """Tags associated with the Fax Application."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    webhook_event_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: Optional[str] = None
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int] = None
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PaginationMetaOAuth"]


class PaginationMetaOAuth(BaseModel):
    page_number: int
    """Current page number"""

    total_pages: int
    """Total number of pages"""

    page_size: Optional[int] = None
    """Number of items per page"""

    total_results: Optional[int] = None
    """Total number of results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AccountRetrieveTranscriptionsJsonParams"]


class AccountRetrieveTranscriptionsJsonParams(TypedDict, total=False):
    page_size: Annotated[int, PropertyInfo(alias="PageSize")]
    """The number of records to be displayed on a page"""

    page_token: Annotated[str, PropertyInfo(alias="PageToken")]
    """Used to request the next page of results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.porting_orders import (
    verification_code_list_params,
    verification_code_send_params,
    verification_code_verify_params,
)
from ...types.porting_orders.verification_code_list_response import VerificationCodeListResponse
from ...types.porting_orders.verification_code_verify_response import VerificationCodeVerifyResponse

__all__ = ["VerificationCodesResource", "AsyncVerificationCodesResource"]


class VerificationCodesResource(SyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> VerificationCodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VerificationCodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VerificationCodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VerificationCodesResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        filter: verification_code_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: verification_code_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[VerificationCodeListResponse]:
        """
        Returns a list of verification codes for a porting order.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[verified]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/verification_codes",
            page=SyncDefaultFlatPagination[VerificationCodeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    verification_code_list_params.VerificationCodeListParams,
                ),
            ),
            model=VerificationCodeListResponse,
        )

    def send(
        self,
        id: str,
        *,
        phone_numbers: SequenceNotStr[str] | Omit = omit,
        verification_method: Literal["sms", "call"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send the verification code for all porting phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/porting_orders/{id}/verification_codes/send",
            body=maybe_transform(
                {
                    "phone_numbers": phone_numbers,
                    "verification_method": verification_method,
                },
                verification_code_send_params.VerificationCodeSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def verify(
        self,
        id: str,
        *,
        verification_codes: Iterable[verification_code_verify_params.VerificationCode] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationCodeVerifyResponse:
        """
        Verifies the verification code for a list of phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/verification_codes/verify",
            body=maybe_transform(
                {"verification_codes": verification_codes}, verification_code_verify_params.VerificationCodeVerifyParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationCodeVerifyResponse,
        )


class AsyncVerificationCodesResource(AsyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AsyncVerificationCodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVerificationCodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVerificationCodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVerificationCodesResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        filter: verification_code_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: verification_code_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VerificationCodeListResponse, AsyncDefaultFlatPagination[VerificationCodeListResponse]]:
        """
        Returns a list of verification codes for a porting order.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[verified]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/verification_codes",
            page=AsyncDefaultFlatPagination[VerificationCodeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    verification_code_list_params.VerificationCodeListParams,
                ),
            ),
            model=VerificationCodeListResponse,
        )

    async def send(
        self,
        id: str,
        *,
        phone_numbers: SequenceNotStr[str] | Omit = omit,
        verification_method: Literal["sms", "call"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send the verification code for all porting phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/porting_orders/{id}/verification_codes/send",
            body=await async_maybe_transform(
                {
                    "phone_numbers": phone_numbers,
                    "verification_method": verification_method,
                },
                verification_code_send_params.VerificationCodeSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def verify(
        self,
        id: str,
        *,
        verification_codes: Iterable[verification_code_verify_params.VerificationCode] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationCodeVerifyResponse:
        """
        Verifies the verification code for a list of phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/verification_codes/verify",
            body=await async_maybe_transform(
                {"verification_codes": verification_codes}, verification_code_verify_params.VerificationCodeVerifyParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationCodeVerifyResponse,
        )


class VerificationCodesResourceWithRawResponse:
    def __init__(self, verification_codes: VerificationCodesResource) -> None:
        self._verification_codes = verification_codes

        self.list = to_raw_response_wrapper(
            verification_codes.list,
        )
        self.send = to_raw_response_wrapper(
            verification_codes.send,
        )
        self.verify = to_raw_response_wrapper(
            verification_codes.verify,
        )


class AsyncVerificationCodesResourceWithRawResponse:
    def __init__(self, verification_codes: AsyncVerificationCodesResource) -> None:
        self._verification_codes = verification_codes

        self.list = async_to_raw_response_wrapper(
            verification_codes.list,
        )
        self.send = async_to_raw_response_wrapper(
            verification_codes.send,
        )
        self.verify = async_to_raw_response_wrapper(
            verification_codes.verify,
        )


class VerificationCodesResourceWithStreamingResponse:
    def __init__(self, verification_codes: VerificationCodesResource) -> None:
        self._verification_codes = verification_codes

        self.list = to_streamed_response_wrapper(
            verification_codes.list,
        )
        self.send = to_streamed_response_wrapper(
            verification_codes.send,
        )
        self.verify = to_streamed_response_wrapper(
            verification_codes.verify,
        )


class AsyncVerificationCodesResourceWithStreamingResponse:
    def __init__(self, verification_codes: AsyncVerificationCodesResource) -> None:
        self._verification_codes = verification_codes

        self.list = async_to_streamed_response_wrapper(
            verification_codes.list,
        )
        self.send = async_to_streamed_response_wrapper(
            verification_codes.send,
        )
        self.verify = async_to_streamed_response_wrapper(
            verification_codes.verify,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ParticipantRetrieveParticipantsResponse", "Participant"]


class Participant(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    api_version: Optional[str] = None
    """The version of the API that was used to make the request."""

    call_sid: Optional[str] = None
    """The identifier of this participant's call."""

    call_sid_legacy: Optional[str] = None
    """The identifier of this participant's call."""

    coaching: Optional[bool] = None
    """Whether the participant is coaching another call."""

    coaching_call_sid: Optional[str] = None
    """The identifier of the coached participant's call."""

    coaching_call_sid_legacy: Optional[str] = None
    """The identifier of the coached participant's call."""

    conference_sid: Optional[str] = None
    """The unique identifier for the conference."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    end_conference_on_exit: Optional[bool] = None
    """Whether the conference ends when the participant leaves."""

    hold: Optional[bool] = None
    """Whether the participant is on hold."""

    muted: Optional[bool] = None
    """Whether the participant is muted."""

    status: Optional[Literal["connecting", "connected", "completed"]] = None
    """The status of the participant's call in the conference."""

    uri: Optional[str] = None
    """The relative URI for this participant."""


class ParticipantRetrieveParticipantsResponse(BaseModel):
    end: Optional[int] = None
    """The number of the last element on the page, zero-indexed."""

    first_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences/6dc6cc1a-1ba1-4351-86b8-4c22c95cd98f/Participants.json?page=0&pagesize=20
    """

    next_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences/6dc6cc1a-1ba1-4351-86b8-4c22c95cd98f/Participants.json?Page=1&PageSize=1&PageToken=MTY4AjgyNDkwNzIxMQ
    """

    page: Optional[int] = None
    """Current page number, zero-indexed."""

    page_size: Optional[int] = None
    """The number of items on the page"""

    participants: Optional[List[Participant]] = None

    start: Optional[int] = None
    """The number of the first element on the page, zero-indexed."""

    uri: Optional[str] = None
    """The URI of the current page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["ActionGenerateJoinClientTokenResponse", "Data"]


class Data(BaseModel):
    token: Optional[str] = None

    refresh_token: Optional[str] = None

    refresh_token_expires_at: Optional[datetime] = None
    """ISO 8601 timestamp when the refresh token expires."""

    token_expires_at: Optional[datetime] = None
    """ISO 8601 timestamp when the token expires."""


class ActionGenerateJoinClientTokenResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import sim_card_order_list_params, sim_card_order_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.sim_card_order import SimCardOrder
from ..types.sim_card_order_create_response import SimCardOrderCreateResponse
from ..types.sim_card_order_retrieve_response import SimCardOrderRetrieveResponse

__all__ = ["SimCardOrdersResource", "AsyncSimCardOrdersResource"]


class SimCardOrdersResource(SyncAPIResource):
    """SIM Card Orders operations"""

    @cached_property
    def with_raw_response(self) -> SimCardOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SimCardOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimCardOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SimCardOrdersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        address_id: str,
        quantity: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderCreateResponse:
        """
        Creates a new order for SIM cards.

        Args:
          address_id: Uniquely identifies the address for the order.

          quantity: The amount of SIM cards to order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_card_orders",
            body=maybe_transform(
                {
                    "address_id": address_id,
                    "quantity": quantity,
                },
                sim_card_order_create_params.SimCardOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderRetrieveResponse:
        """
        Get a single SIM card order by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_card_orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: sim_card_order_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[SimCardOrder]:
        """
        Get all SIM card orders according to filters.

        Args:
          filter: Consolidated filter parameter for SIM card orders (deepObject style).
              Originally: filter[created_at], filter[updated_at], filter[quantity],
              filter[cost.amount], filter[cost.currency], filter[address.id],
              filter[address.street_address], filter[address.extended_address],
              filter[address.locality], filter[address.administrative_area],
              filter[address.country_code], filter[address.postal_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_orders",
            page=SyncDefaultFlatPagination[SimCardOrder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_order_list_params.SimCardOrderListParams,
                ),
            ),
            model=SimCardOrder,
        )


class AsyncSimCardOrdersResource(AsyncAPIResource):
    """SIM Card Orders operations"""

    @cached_property
    def with_raw_response(self) -> AsyncSimCardOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimCardOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimCardOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSimCardOrdersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        address_id: str,
        quantity: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderCreateResponse:
        """
        Creates a new order for SIM cards.

        Args:
          address_id: Uniquely identifies the address for the order.

          quantity: The amount of SIM cards to order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_card_orders",
            body=await async_maybe_transform(
                {
                    "address_id": address_id,
                    "quantity": quantity,
                },
                sim_card_order_create_params.SimCardOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderRetrieveResponse:
        """
        Get a single SIM card order by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_card_orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: sim_card_order_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimCardOrder, AsyncDefaultFlatPagination[SimCardOrder]]:
        """
        Get all SIM card orders according to filters.

        Args:
          filter: Consolidated filter parameter for SIM card orders (deepObject style).
              Originally: filter[created_at], filter[updated_at], filter[quantity],
              filter[cost.amount], filter[cost.currency], filter[address.id],
              filter[address.street_address], filter[address.extended_address],
              filter[address.locality], filter[address.administrative_area],
              filter[address.country_code], filter[address.postal_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_orders",
            page=AsyncDefaultFlatPagination[SimCardOrder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_order_list_params.SimCardOrderListParams,
                ),
            ),
            model=SimCardOrder,
        )


class SimCardOrdersResourceWithRawResponse:
    def __init__(self, sim_card_orders: SimCardOrdersResource) -> None:
        self._sim_card_orders = sim_card_orders

        self.create = to_raw_response_wrapper(
            sim_card_orders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            sim_card_orders.retrieve,
        )
        self.list = to_raw_response_wrapper(
            sim_card_orders.list,
        )


class AsyncSimCardOrdersResourceWithRawResponse:
    def __init__(self, sim_card_orders: AsyncSimCardOrdersResource) -> None:
        self._sim_card_orders = sim_card_orders

        self.create = async_to_raw_response_wrapper(
            sim_card_orders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            sim_card_orders.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            sim_card_orders.list,
        )


class SimCardOrdersResourceWithStreamingResponse:
    def __init__(self, sim_card_orders: SimCardOrdersResource) -> None:
        self._sim_card_orders = sim_card_orders

        self.create = to_streamed_response_wrapper(
            sim_card_orders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            sim_card_orders.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            sim_card_orders.list,
        )


class AsyncSimCardOrdersResourceWithStreamingResponse:
    def __init__(self, sim_card_orders: AsyncSimCardOrdersResource) -> None:
        self._sim_card_orders = sim_card_orders

        self.create = async_to_streamed_response_wrapper(
            sim_card_orders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            sim_card_orders.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            sim_card_orders.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import global_ip_assignments_usage_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.global_ip_assignments_usage_retrieve_response import GlobalIPAssignmentsUsageRetrieveResponse

__all__ = ["GlobalIPAssignmentsUsageResource", "AsyncGlobalIPAssignmentsUsageResource"]


class GlobalIPAssignmentsUsageResource(SyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> GlobalIPAssignmentsUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return GlobalIPAssignmentsUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalIPAssignmentsUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return GlobalIPAssignmentsUsageResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        filter: global_ip_assignments_usage_retrieve_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentsUsageRetrieveResponse:
        """
        Global IP Assignment Usage Metrics

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[global_ip_assignment_id][in], filter[global_ip_id][in]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/global_ip_assignments_usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"filter": filter},
                    global_ip_assignments_usage_retrieve_params.GlobalIPAssignmentsUsageRetrieveParams,
                ),
            ),
            cast_to=GlobalIPAssignmentsUsageRetrieveResponse,
        )


class AsyncGlobalIPAssignmentsUsageResource(AsyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> AsyncGlobalIPAssignmentsUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalIPAssignmentsUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalIPAssignmentsUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncGlobalIPAssignmentsUsageResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        filter: global_ip_assignments_usage_retrieve_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentsUsageRetrieveResponse:
        """
        Global IP Assignment Usage Metrics

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[global_ip_assignment_id][in], filter[global_ip_id][in]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/global_ip_assignments_usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter},
                    global_ip_assignments_usage_retrieve_params.GlobalIPAssignmentsUsageRetrieveParams,
                ),
            ),
            cast_to=GlobalIPAssignmentsUsageRetrieveResponse,
        )


class GlobalIPAssignmentsUsageResourceWithRawResponse:
    def __init__(self, global_ip_assignments_usage: GlobalIPAssignmentsUsageResource) -> None:
        self._global_ip_assignments_usage = global_ip_assignments_usage

        self.retrieve = to_raw_response_wrapper(
            global_ip_assignments_usage.retrieve,
        )


class AsyncGlobalIPAssignmentsUsageResourceWithRawResponse:
    def __init__(self, global_ip_assignments_usage: AsyncGlobalIPAssignmentsUsageResource) -> None:
        self._global_ip_assignments_usage = global_ip_assignments_usage

        self.retrieve = async_to_raw_response_wrapper(
            global_ip_assignments_usage.retrieve,
        )


class GlobalIPAssignmentsUsageResourceWithStreamingResponse:
    def __init__(self, global_ip_assignments_usage: GlobalIPAssignmentsUsageResource) -> None:
        self._global_ip_assignments_usage = global_ip_assignments_usage

        self.retrieve = to_streamed_response_wrapper(
            global_ip_assignments_usage.retrieve,
        )


class AsyncGlobalIPAssignmentsUsageResourceWithStreamingResponse:
    def __init__(self, global_ip_assignments_usage: AsyncGlobalIPAssignmentsUsageResource) -> None:
        self._global_ip_assignments_usage = global_ip_assignments_usage

        self.retrieve = async_to_streamed_response_wrapper(
            global_ip_assignments_usage.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TranscriptionConfigParam"]


class TranscriptionConfigParam(TypedDict, total=False):
    """The settings associated with speech to text for the voice assistant.

    This is only relevant if the assistant uses a text-to-text language model. Any assistant using a model with native audio support (e.g. `fixie-ai/ultravox-v0_4`) will ignore this field.
    """

    model: str
    """The speech to text model to be used by the voice assistant.

    - `distil-whisper/distil-large-v2` is lower latency but English-only.
    - `openai/whisper-large-v3-turbo` is multi-lingual with automatic language
      detection but slightly higher latency.
    - `google` is a multi-lingual option, please describe the language in the
      `language` field.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionVerifyParams"]


class ActionVerifyParams(TypedDict, total=False):
    code: str
    """This is the code the user submits for verification."""

    status: Literal["accepted", "rejected"]
    """Identifies if the verification code has been accepted or rejected.

    Only permitted if custom_code was used for the verification.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .http import HTTP
from .._models import BaseModel

__all__ = ["Attempt"]


class Attempt(BaseModel):
    """Webhook delivery attempt details."""

    errors: Optional[List[int]] = None
    """Webhook delivery error codes."""

    finished_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the attempt has finished."""

    http: Optional[HTTP] = None
    """HTTP request and response information."""

    started_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the attempt was initiated."""

    status: Optional[Literal["delivered", "failed"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AwsVoiceSettingsParam"]


class AwsVoiceSettingsParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    type: Required[Literal["aws"]]
    """Voice settings provider type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    customer_service_record_list_params,
    customer_service_record_create_params,
    customer_service_record_verify_phone_number_coverage_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.customer_service_record import CustomerServiceRecord
from ..types.customer_service_record_create_response import CustomerServiceRecordCreateResponse
from ..types.customer_service_record_retrieve_response import CustomerServiceRecordRetrieveResponse
from ..types.customer_service_record_verify_phone_number_coverage_response import (
    CustomerServiceRecordVerifyPhoneNumberCoverageResponse,
)

__all__ = ["CustomerServiceRecordsResource", "AsyncCustomerServiceRecordsResource"]


class CustomerServiceRecordsResource(SyncAPIResource):
    """Customer Service Record operations"""

    @cached_property
    def with_raw_response(self) -> CustomerServiceRecordsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CustomerServiceRecordsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomerServiceRecordsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CustomerServiceRecordsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        phone_number: str,
        additional_data: customer_service_record_create_params.AdditionalData | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordCreateResponse:
        """
        Create a new customer service record for the provided phone number.

        Args:
          phone_number: A valid US phone number in E164 format.

          webhook_url: Callback URL to receive webhook notifications.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer_service_records",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "additional_data": additional_data,
                    "webhook_url": webhook_url,
                },
                customer_service_record_create_params.CustomerServiceRecordCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordCreateResponse,
        )

    def retrieve(
        self,
        customer_service_record_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordRetrieveResponse:
        """
        Get a specific customer service record.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_service_record_id:
            raise ValueError(
                f"Expected a non-empty value for `customer_service_record_id` but received {customer_service_record_id!r}"
            )
        return self._get(
            f"/customer_service_records/{customer_service_record_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: customer_service_record_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: customer_service_record_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[CustomerServiceRecord]:
        """
        List customer service records.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number][eq], filter[phone_number][in][], filter[status][eq],
              filter[status][in][], filter[created_at][lt], filter[created_at][gt]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/customer_service_records",
            page=SyncDefaultFlatPagination[CustomerServiceRecord],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    customer_service_record_list_params.CustomerServiceRecordListParams,
                ),
            ),
            model=CustomerServiceRecord,
        )

    def verify_phone_number_coverage(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordVerifyPhoneNumberCoverageResponse:
        """
        Verify the coverage for a list of phone numbers.

        Args:
          phone_numbers: The phone numbers list to be verified.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer_service_records/phone_number_coverages",
            body=maybe_transform(
                {"phone_numbers": phone_numbers},
                customer_service_record_verify_phone_number_coverage_params.CustomerServiceRecordVerifyPhoneNumberCoverageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordVerifyPhoneNumberCoverageResponse,
        )


class AsyncCustomerServiceRecordsResource(AsyncAPIResource):
    """Customer Service Record operations"""

    @cached_property
    def with_raw_response(self) -> AsyncCustomerServiceRecordsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomerServiceRecordsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomerServiceRecordsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCustomerServiceRecordsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        phone_number: str,
        additional_data: customer_service_record_create_params.AdditionalData | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordCreateResponse:
        """
        Create a new customer service record for the provided phone number.

        Args:
          phone_number: A valid US phone number in E164 format.

          webhook_url: Callback URL to receive webhook notifications.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer_service_records",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "additional_data": additional_data,
                    "webhook_url": webhook_url,
                },
                customer_service_record_create_params.CustomerServiceRecordCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordCreateResponse,
        )

    async def retrieve(
        self,
        customer_service_record_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordRetrieveResponse:
        """
        Get a specific customer service record.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_service_record_id:
            raise ValueError(
                f"Expected a non-empty value for `customer_service_record_id` but received {customer_service_record_id!r}"
            )
        return await self._get(
            f"/customer_service_records/{customer_service_record_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: customer_service_record_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: customer_service_record_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerServiceRecord, AsyncDefaultFlatPagination[CustomerServiceRecord]]:
        """
        List customer service records.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number][eq], filter[phone_number][in][], filter[status][eq],
              filter[status][in][], filter[created_at][lt], filter[created_at][gt]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/customer_service_records",
            page=AsyncDefaultFlatPagination[CustomerServiceRecord],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    customer_service_record_list_params.CustomerServiceRecordListParams,
                ),
            ),
            model=CustomerServiceRecord,
        )

    async def verify_phone_number_coverage(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerServiceRecordVerifyPhoneNumberCoverageResponse:
        """
        Verify the coverage for a list of phone numbers.

        Args:
          phone_numbers: The phone numbers list to be verified.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer_service_records/phone_number_coverages",
            body=await async_maybe_transform(
                {"phone_numbers": phone_numbers},
                customer_service_record_verify_phone_number_coverage_params.CustomerServiceRecordVerifyPhoneNumberCoverageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerServiceRecordVerifyPhoneNumberCoverageResponse,
        )


class CustomerServiceRecordsResourceWithRawResponse:
    def __init__(self, customer_service_records: CustomerServiceRecordsResource) -> None:
        self._customer_service_records = customer_service_records

        self.create = to_raw_response_wrapper(
            customer_service_records.create,
        )
        self.retrieve = to_raw_response_wrapper(
            customer_service_records.retrieve,
        )
        self.list = to_raw_response_wrapper(
            customer_service_records.list,
        )
        self.verify_phone_number_coverage = to_raw_response_wrapper(
            customer_service_records.verify_phone_number_coverage,
        )


class AsyncCustomerServiceRecordsResourceWithRawResponse:
    def __init__(self, customer_service_records: AsyncCustomerServiceRecordsResource) -> None:
        self._customer_service_records = customer_service_records

        self.create = async_to_raw_response_wrapper(
            customer_service_records.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            customer_service_records.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            customer_service_records.list,
        )
        self.verify_phone_number_coverage = async_to_raw_response_wrapper(
            customer_service_records.verify_phone_number_coverage,
        )


class CustomerServiceRecordsResourceWithStreamingResponse:
    def __init__(self, customer_service_records: CustomerServiceRecordsResource) -> None:
        self._customer_service_records = customer_service_records

        self.create = to_streamed_response_wrapper(
            customer_service_records.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            customer_service_records.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            customer_service_records.list,
        )
        self.verify_phone_number_coverage = to_streamed_response_wrapper(
            customer_service_records.verify_phone_number_coverage,
        )


class AsyncCustomerServiceRecordsResourceWithStreamingResponse:
    def __init__(self, customer_service_records: AsyncCustomerServiceRecordsResource) -> None:
        self._customer_service_records = customer_service_records

        self.create = async_to_streamed_response_wrapper(
            customer_service_records.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            customer_service_records.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            customer_service_records.list,
        )
        self.verify_phone_number_coverage = async_to_streamed_response_wrapper(
            customer_service_records.verify_phone_number_coverage,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .brand import (
    BrandResource,
    AsyncBrandResource,
    BrandResourceWithRawResponse,
    AsyncBrandResourceWithRawResponse,
    BrandResourceWithStreamingResponse,
    AsyncBrandResourceWithStreamingResponse,
)
from .campaign import (
    CampaignResource,
    AsyncCampaignResource,
    CampaignResourceWithRawResponse,
    AsyncCampaignResourceWithRawResponse,
    CampaignResourceWithStreamingResponse,
    AsyncCampaignResourceWithStreamingResponse,
)
from .messaging_10dlc import (
    Messaging10dlcResource,
    AsyncMessaging10dlcResource,
    Messaging10dlcResourceWithRawResponse,
    AsyncMessaging10dlcResourceWithRawResponse,
    Messaging10dlcResourceWithStreamingResponse,
    AsyncMessaging10dlcResourceWithStreamingResponse,
)
from .campaign_builder import (
    CampaignBuilderResource,
    AsyncCampaignBuilderResource,
    CampaignBuilderResourceWithRawResponse,
    AsyncCampaignBuilderResourceWithRawResponse,
    CampaignBuilderResourceWithStreamingResponse,
    AsyncCampaignBuilderResourceWithStreamingResponse,
)
from .partner_campaigns import (
    PartnerCampaignsResource,
    AsyncPartnerCampaignsResource,
    PartnerCampaignsResourceWithRawResponse,
    AsyncPartnerCampaignsResourceWithRawResponse,
    PartnerCampaignsResourceWithStreamingResponse,
    AsyncPartnerCampaignsResourceWithStreamingResponse,
)
from .phone_number_campaigns import (
    PhoneNumberCampaignsResource,
    AsyncPhoneNumberCampaignsResource,
    PhoneNumberCampaignsResourceWithRawResponse,
    AsyncPhoneNumberCampaignsResourceWithRawResponse,
    PhoneNumberCampaignsResourceWithStreamingResponse,
    AsyncPhoneNumberCampaignsResourceWithStreamingResponse,
)
from .phone_number_assignment_by_profile import (
    PhoneNumberAssignmentByProfileResource,
    AsyncPhoneNumberAssignmentByProfileResource,
    PhoneNumberAssignmentByProfileResourceWithRawResponse,
    AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse,
    PhoneNumberAssignmentByProfileResourceWithStreamingResponse,
    AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse,
)

__all__ = [
    "BrandResource",
    "AsyncBrandResource",
    "BrandResourceWithRawResponse",
    "AsyncBrandResourceWithRawResponse",
    "BrandResourceWithStreamingResponse",
    "AsyncBrandResourceWithStreamingResponse",
    "CampaignResource",
    "AsyncCampaignResource",
    "CampaignResourceWithRawResponse",
    "AsyncCampaignResourceWithRawResponse",
    "CampaignResourceWithStreamingResponse",
    "AsyncCampaignResourceWithStreamingResponse",
    "CampaignBuilderResource",
    "AsyncCampaignBuilderResource",
    "CampaignBuilderResourceWithRawResponse",
    "AsyncCampaignBuilderResourceWithRawResponse",
    "CampaignBuilderResourceWithStreamingResponse",
    "AsyncCampaignBuilderResourceWithStreamingResponse",
    "PartnerCampaignsResource",
    "AsyncPartnerCampaignsResource",
    "PartnerCampaignsResourceWithRawResponse",
    "AsyncPartnerCampaignsResourceWithRawResponse",
    "PartnerCampaignsResourceWithStreamingResponse",
    "AsyncPartnerCampaignsResourceWithStreamingResponse",
    "PhoneNumberCampaignsResource",
    "AsyncPhoneNumberCampaignsResource",
    "PhoneNumberCampaignsResourceWithRawResponse",
    "AsyncPhoneNumberCampaignsResourceWithRawResponse",
    "PhoneNumberCampaignsResourceWithStreamingResponse",
    "AsyncPhoneNumberCampaignsResourceWithStreamingResponse",
    "PhoneNumberAssignmentByProfileResource",
    "AsyncPhoneNumberAssignmentByProfileResource",
    "PhoneNumberAssignmentByProfileResourceWithRawResponse",
    "AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse",
    "PhoneNumberAssignmentByProfileResourceWithStreamingResponse",
    "AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse",
    "Messaging10dlcResource",
    "AsyncMessaging10dlcResource",
    "Messaging10dlcResourceWithRawResponse",
    "AsyncMessaging10dlcResourceWithRawResponse",
    "Messaging10dlcResourceWithStreamingResponse",
    "AsyncMessaging10dlcResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["StockExchange"]

StockExchange: TypeAlias = Literal[
    "NONE",
    "NASDAQ",
    "NYSE",
    "AMEX",
    "AMX",
    "ASX",
    "B3",
    "BME",
    "BSE",
    "FRA",
    "ICEX",
    "JPX",
    "JSE",
    "KRX",
    "LON",
    "NSE",
    "OMX",
    "SEHK",
    "SSE",
    "STO",
    "SWX",
    "SZSE",
    "TSX",
    "TWSE",
    "VSE",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from .plan import (
    PlanResource,
    AsyncPlanResource,
    PlanResourceWithRawResponse,
    AsyncPlanResourceWithRawResponse,
    PlanResourceWithStreamingResponse,
    AsyncPlanResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .telnyx_agents import (
    TelnyxAgentsResource,
    AsyncTelnyxAgentsResource,
    TelnyxAgentsResourceWithRawResponse,
    AsyncTelnyxAgentsResourceWithRawResponse,
    TelnyxAgentsResourceWithStreamingResponse,
    AsyncTelnyxAgentsResourceWithStreamingResponse,
)
from .....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ....._base_client import AsyncPaginator, make_request_options
from .....types.ai.missions import run_list_params, run_create_params, run_update_params, run_list_runs_params
from .....types.ai.missions.mission_run_data import MissionRunData
from .....types.ai.missions.run_create_response import RunCreateResponse
from .....types.ai.missions.run_update_response import RunUpdateResponse
from .....types.ai.missions.run_retrieve_response import RunRetrieveResponse
from .....types.ai.missions.run_pause_run_response import RunPauseRunResponse
from .....types.ai.missions.run_cancel_run_response import RunCancelRunResponse
from .....types.ai.missions.run_resume_run_response import RunResumeRunResponse

__all__ = ["RunsResource", "AsyncRunsResource"]


class RunsResource(SyncAPIResource):
    @cached_property
    def events(self) -> EventsResource:
        return EventsResource(self._client)

    @cached_property
    def plan(self) -> PlanResource:
        return PlanResource(self._client)

    @cached_property
    def telnyx_agents(self) -> TelnyxAgentsResource:
        return TelnyxAgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RunsResourceWithStreamingResponse(self)

    def create(
        self,
        mission_id: str,
        *,
        input: Dict[str, object] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCreateResponse:
        """
        Start a new run for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs",
            body=maybe_transform(
                {
                    "input": input,
                    "metadata": metadata,
                },
                run_create_params.RunCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCreateResponse,
        )

    def retrieve(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunRetrieveResponse:
        """
        Get details of a specific run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunRetrieveResponse,
        )

    def update(
        self,
        run_id: str,
        *,
        mission_id: str,
        error: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        result_payload: Dict[str, object] | Omit = omit,
        result_summary: str | Omit = omit,
        status: Literal["pending", "running", "paused", "succeeded", "failed", "cancelled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunUpdateResponse:
        """
        Update run status and/or result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._patch(
            f"/ai/missions/{mission_id}/runs/{run_id}",
            body=maybe_transform(
                {
                    "error": error,
                    "metadata": metadata,
                    "result_payload": result_payload,
                    "result_summary": result_summary,
                    "status": status,
                },
                run_update_params.RunUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunUpdateResponse,
        )

    def list(
        self,
        mission_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MissionRunData]:
        """
        List all runs for a specific mission

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._get_api_list(
            f"/ai/missions/{mission_id}/runs",
            page=SyncDefaultFlatPagination[MissionRunData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    run_list_params.RunListParams,
                ),
            ),
            model=MissionRunData,
        )

    def cancel_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCancelRunResponse:
        """
        Cancel a running or paused run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCancelRunResponse,
        )

    def list_runs(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MissionRunData]:
        """
        List recent runs across all missions

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions/runs",
            page=SyncDefaultFlatPagination[MissionRunData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    run_list_runs_params.RunListRunsParams,
                ),
            ),
            model=MissionRunData,
        )

    def pause_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunPauseRunResponse:
        """
        Pause a running run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/pause",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunPauseRunResponse,
        )

    def resume_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunResumeRunResponse:
        """
        Resume a paused run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/resume",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunResumeRunResponse,
        )


class AsyncRunsResource(AsyncAPIResource):
    @cached_property
    def events(self) -> AsyncEventsResource:
        return AsyncEventsResource(self._client)

    @cached_property
    def plan(self) -> AsyncPlanResource:
        return AsyncPlanResource(self._client)

    @cached_property
    def telnyx_agents(self) -> AsyncTelnyxAgentsResource:
        return AsyncTelnyxAgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRunsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRunsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRunsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRunsResourceWithStreamingResponse(self)

    async def create(
        self,
        mission_id: str,
        *,
        input: Dict[str, object] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCreateResponse:
        """
        Start a new run for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "metadata": metadata,
                },
                run_create_params.RunCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCreateResponse,
        )

    async def retrieve(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunRetrieveResponse:
        """
        Get details of a specific run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunRetrieveResponse,
        )

    async def update(
        self,
        run_id: str,
        *,
        mission_id: str,
        error: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        result_payload: Dict[str, object] | Omit = omit,
        result_summary: str | Omit = omit,
        status: Literal["pending", "running", "paused", "succeeded", "failed", "cancelled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunUpdateResponse:
        """
        Update run status and/or result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._patch(
            f"/ai/missions/{mission_id}/runs/{run_id}",
            body=await async_maybe_transform(
                {
                    "error": error,
                    "metadata": metadata,
                    "result_payload": result_payload,
                    "result_summary": result_summary,
                    "status": status,
                },
                run_update_params.RunUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunUpdateResponse,
        )

    def list(
        self,
        mission_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MissionRunData, AsyncDefaultFlatPagination[MissionRunData]]:
        """
        List all runs for a specific mission

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._get_api_list(
            f"/ai/missions/{mission_id}/runs",
            page=AsyncDefaultFlatPagination[MissionRunData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    run_list_params.RunListParams,
                ),
            ),
            model=MissionRunData,
        )

    async def cancel_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunCancelRunResponse:
        """
        Cancel a running or paused run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunCancelRunResponse,
        )

    def list_runs(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MissionRunData, AsyncDefaultFlatPagination[MissionRunData]]:
        """
        List recent runs across all missions

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions/runs",
            page=AsyncDefaultFlatPagination[MissionRunData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    run_list_runs_params.RunListRunsParams,
                ),
            ),
            model=MissionRunData,
        )

    async def pause_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunPauseRunResponse:
        """
        Pause a running run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/pause",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunPauseRunResponse,
        )

    async def resume_run(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunResumeRunResponse:
        """
        Resume a paused run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/resume",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunResumeRunResponse,
        )


class RunsResourceWithRawResponse:
    def __init__(self, runs: RunsResource) -> None:
        self._runs = runs

        self.create = to_raw_response_wrapper(
            runs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            runs.retrieve,
        )
        self.update = to_raw_response_wrapper(
            runs.update,
        )
        self.list = to_raw_response_wrapper(
            runs.list,
        )
        self.cancel_run = to_raw_response_wrapper(
            runs.cancel_run,
        )
        self.list_runs = to_raw_response_wrapper(
            runs.list_runs,
        )
        self.pause_run = to_raw_response_wrapper(
            runs.pause_run,
        )
        self.resume_run = to_raw_response_wrapper(
            runs.resume_run,
        )

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        return EventsResourceWithRawResponse(self._runs.events)

    @cached_property
    def plan(self) -> PlanResourceWithRawResponse:
        return PlanResourceWithRawResponse(self._runs.plan)

    @cached_property
    def telnyx_agents(self) -> TelnyxAgentsResourceWithRawResponse:
        return TelnyxAgentsResourceWithRawResponse(self._runs.telnyx_agents)


class AsyncRunsResourceWithRawResponse:
    def __init__(self, runs: AsyncRunsResource) -> None:
        self._runs = runs

        self.create = async_to_raw_response_wrapper(
            runs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            runs.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            runs.update,
        )
        self.list = async_to_raw_response_wrapper(
            runs.list,
        )
        self.cancel_run = async_to_raw_response_wrapper(
            runs.cancel_run,
        )
        self.list_runs = async_to_raw_response_wrapper(
            runs.list_runs,
        )
        self.pause_run = async_to_raw_response_wrapper(
            runs.pause_run,
        )
        self.resume_run = async_to_raw_response_wrapper(
            runs.resume_run,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        return AsyncEventsResourceWithRawResponse(self._runs.events)

    @cached_property
    def plan(self) -> AsyncPlanResourceWithRawResponse:
        return AsyncPlanResourceWithRawResponse(self._runs.plan)

    @cached_property
    def telnyx_agents(self) -> AsyncTelnyxAgentsResourceWithRawResponse:
        return AsyncTelnyxAgentsResourceWithRawResponse(self._runs.telnyx_agents)


class RunsResourceWithStreamingResponse:
    def __init__(self, runs: RunsResource) -> None:
        self._runs = runs

        self.create = to_streamed_response_wrapper(
            runs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            runs.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            runs.update,
        )
        self.list = to_streamed_response_wrapper(
            runs.list,
        )
        self.cancel_run = to_streamed_response_wrapper(
            runs.cancel_run,
        )
        self.list_runs = to_streamed_response_wrapper(
            runs.list_runs,
        )
        self.pause_run = to_streamed_response_wrapper(
            runs.pause_run,
        )
        self.resume_run = to_streamed_response_wrapper(
            runs.resume_run,
        )

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        return EventsResourceWithStreamingResponse(self._runs.events)

    @cached_property
    def plan(self) -> PlanResourceWithStreamingResponse:
        return PlanResourceWithStreamingResponse(self._runs.plan)

    @cached_property
    def telnyx_agents(self) -> TelnyxAgentsResourceWithStreamingResponse:
        return TelnyxAgentsResourceWithStreamingResponse(self._runs.telnyx_agents)


class AsyncRunsResourceWithStreamingResponse:
    def __init__(self, runs: AsyncRunsResource) -> None:
        self._runs = runs

        self.create = async_to_streamed_response_wrapper(
            runs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            runs.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            runs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            runs.list,
        )
        self.cancel_run = async_to_streamed_response_wrapper(
            runs.cancel_run,
        )
        self.list_runs = async_to_streamed_response_wrapper(
            runs.list_runs,
        )
        self.pause_run = async_to_streamed_response_wrapper(
            runs.pause_run,
        )
        self.resume_run = async_to_streamed_response_wrapper(
            runs.resume_run,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        return AsyncEventsResourceWithStreamingResponse(self._runs.events)

    @cached_property
    def plan(self) -> AsyncPlanResourceWithStreamingResponse:
        return AsyncPlanResourceWithStreamingResponse(self._runs.plan)

    @cached_property
    def telnyx_agents(self) -> AsyncTelnyxAgentsResourceWithStreamingResponse:
        return AsyncTelnyxAgentsResourceWithStreamingResponse(self._runs.telnyx_agents)
